@@ -194,6 +194,13 @@ public abstract class Launcher {
             return commands;
         }
 
+        /**
+         * Hide parts of the command line from being printed to the log.
+         * @param masks true for each position in {@link #cmds(String[])} which should be masked, false to print
+         * @return this
+         * @see ArgumentListBuilder#add(String, boolean)
+         * @see #maskedPrintCommandLine(List, boolean[], FilePath)
+         */
         public ProcStarter masks(boolean... masks) {
             this.masks = masks;
             return this;
@@ -271,7 +278,8 @@ public abstract class Launcher {
          * becomes the "current" process), these variables will be also set.
          */
         public ProcStarter envs(Map<String, String> overrides) {
-            return envs(Util.mapToEnv(overrides));
+            this.envs = Util.mapToEnv(overrides);
+            return this;
         }
 
         /**
@@ -279,12 +287,19 @@ public abstract class Launcher {
          *      List of "VAR=VALUE". See {@link #envs(Map)} for the semantics.
          */
         public ProcStarter envs(String... overrides) {
+            if (overrides != null) {
+                for (String override : overrides) {
+                    if (override.indexOf('=') == -1) {
+                        throw new IllegalArgumentException(override);
+                    }
+                }
+            }
             this.envs = overrides;
             return this;
         }
 
         public String[] envs() {
-            return envs;
+            return envs.clone();
         }
 
         /**
@@ -683,6 +698,45 @@ public abstract class Launcher {
         };
     }
 
+    /**
+     * Returns a decorated {@link Launcher} that automatically adds the specified environment
+     * variables.
+     *
+     * Those that are specified in {@link ProcStarter#envs(String...)} will take precedence over
+     * what's specified here.
+     *
+     * @since 1.489
+     */
+    public final Launcher decorateByEnv(EnvVars _env) {
+        final EnvVars env = new EnvVars(_env);
+        final Launcher outer = this;
+        return new Launcher(outer) {
+            @Override
+            public Proc launch(ProcStarter starter) throws IOException {
+                EnvVars e = new EnvVars(env);
+                if (starter.envs!=null) {
+                    for (String env : starter.envs) {
+                        e.addLine(env);
+                    }
+                }
+                starter.envs = Util.mapToEnv(e);
+                return outer.launch(starter);
+            }
+
+            @Override
+            public Channel launchChannel(String[] cmd, OutputStream out, FilePath workDir, Map<String, String> envVars) throws IOException, InterruptedException {
+                EnvVars e = new EnvVars(env);
+                e.putAll(envVars);
+                return outer.launchChannel(cmd,out,workDir,e);
+            }
+
+            @Override
+            public void kill(Map<String, String> modelEnvVars) throws IOException, InterruptedException {
+                outer.kill(modelEnvVars);
+            }
+        };
+    }
+
     /**
      * {@link Launcher} that launches process locally.
      */
