@@ -26,12 +26,14 @@ package hudson;
 import com.sun.jna.Memory;
 import com.sun.jna.Native;
 import com.sun.jna.NativeLong;
+import edu.umd.cs.findbugs.annotations.SuppressWarnings;
 import hudson.Proc.LocalProc;
 import hudson.model.TaskListener;
 import hudson.os.PosixAPI;
 import hudson.util.IOException2;
 import hudson.util.QuotedStringTokenizer;
 import hudson.util.VariableResolver;
+import hudson.util.jna.WinIOException;
 import jenkins.model.Jenkins;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.time.FastDateFormat;
@@ -48,6 +50,8 @@ import org.kohsuke.stapler.Stapler;
 import javax.crypto.SecretKey;
 import javax.crypto.spec.SecretKeySpec;
 import java.io.*;
+import java.lang.reflect.Array;
+import java.lang.reflect.InvocationTargetException;
 import java.net.InetAddress;
 import java.net.URI;
 import java.net.URISyntaxException;
@@ -63,6 +67,7 @@ import java.security.NoSuchAlgorithmException;
 import java.text.NumberFormat;
 import java.text.ParseException;
 import java.util.*;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import java.util.regex.Matcher;
@@ -231,6 +236,18 @@ public class Util {
 
             if(!f.delete() && f.exists()) {
                 // trouble-shooting.
+                try {
+                    Class.forName("java.nio.file.Files").getMethod("delete", Class.forName("java.nio.file.Path")).invoke(null, File.class.getMethod("toPath").invoke(f));
+                } catch (InvocationTargetException x) {
+                    Throwable x2 = x.getCause();
+                    if (x2 instanceof IOException) {
+                        // may have a specific exception message
+                        throw (IOException) x2;
+                    }
+                    // else suppress
+                } catch (Throwable x) {
+                    // linkage errors, etc.; suppress
+                }
                 // see http://www.nabble.com/Sometimes-can%27t-delete-files-from-hudson.scm.SubversionSCM%24CheckOutTask.invoke%28%29-tt17333292.html
                 // I suspect other processes putting files in this directory
                 File[] files = f.listFiles();
@@ -246,6 +263,15 @@ public class Util {
      */
     @IgnoreJRERequirement
     private static void makeWritable(File f) {
+        // try JDK6-way of doing it.
+        try {
+            if (f.setWritable(true)) {
+                return;
+            }
+        } catch (NoSuchMethodError e) {
+            // not JDK6
+        }
+
         // try chmod. this becomes no-op if this is not Unix.
         try {
             Chmod chmod = new Chmod();
@@ -257,13 +283,6 @@ public class Util {
             LOGGER.log(Level.INFO,"Failed to chmod "+f,e);
         }
 
-        // also try JDK6-way of doing it.
-        try {
-            f.setWritable(true);
-        } catch (NoSuchMethodError e) {
-            // not JDK6
-        }
-
         try {// try libc chmod
             POSIX posix = PosixAPI.get();
             String path = f.getAbsolutePath();
@@ -310,8 +329,18 @@ public class Util {
      */
     //Taken from http://svn.apache.org/viewvc/maven/shared/trunk/file-management/src/main/java/org/apache/maven/shared/model/fileset/util/FileSetManager.java?view=markup
     public static boolean isSymlink(File file) throws IOException {
+        Boolean r = isSymlinkJava7(file);
+        if (r != null) {
+            return r;
+        }
         if (Functions.isWindows()) {
-          return Kernel32Utils.isJunctionOrSymlink(file);
+            try {
+                return Kernel32Utils.isJunctionOrSymlink(file);
+            } catch (UnsupportedOperationException e) {
+                // fall through
+            } catch (LinkageError e) {
+                // fall through
+            }
         }
         String name = file.getName();
         if (name.equals(".") || name.equals(".."))
@@ -327,6 +356,18 @@ public class Util {
         return !fileInCanonicalParent.getCanonicalFile().equals( fileInCanonicalParent.getAbsoluteFile() );
     }
 
+    @SuppressWarnings("NP_BOOLEAN_RETURN_NULL")
+    private static Boolean isSymlinkJava7(File file) throws IOException {
+        try {
+            Object path = File.class.getMethod("toPath").invoke(file);
+            return (Boolean) Class.forName("java.nio.file.Files").getMethod("isSymbolicLink", Class.forName("java.nio.file.Path")).invoke(null, path);
+        } catch (NoSuchMethodException x) {
+            return null; // fine, Java 5/6
+        } catch (Exception x) {
+            throw (IOException) new IOException(x.toString()).initCause(x);
+        }
+    }
+
     /**
      * Creates a new temporary directory.
      */
@@ -487,6 +528,25 @@ public class Util {
         else                return s;
     }
 
+
+    /**
+     * Ensure string ends with suffix
+     *
+     * @param subject Examined string
+     * @param suffix  Desired suffix
+     * @return Original subject in case it already ends with suffix, null in
+     *         case subject was null and subject + suffix otherwise.
+     * @since 1.505
+     */
+    public static String ensureEndsWith(String subject, String suffix) {
+
+        if (subject == null) return null;
+
+        if (subject.endsWith(suffix)) return subject;
+
+        return subject + suffix;
+    }
+
     /**
      * Computes MD5 digest of the given input stream.
      *
@@ -683,7 +743,7 @@ public class Util {
 
             for (int i = 0; i < s.length(); i++) {
                 int c = s.charAt(i);
-                if (c<128 && (c!=' ' && c!='&')) {
+                if (c<128 && c!=' ') {
                     out.append((char) c);
                 } else {
                     // 1 char -> UTF8
@@ -708,8 +768,8 @@ public class Util {
     private static final boolean[] uriMap = new boolean[123];
     static {
         String raw =
-    "!  $  '()*+,-. 0123456789   =  @ABCDEFGHIJKLMNOPQRSTUVWXYZ    _ abcdefghijklmnopqrstuvwxyz";
-  //  "# %&        /          :;< >?                           [\]^ `                          {|}~
+    "!  $ &'()*+,-. 0123456789   =  @ABCDEFGHIJKLMNOPQRSTUVWXYZ    _ abcdefghijklmnopqrstuvwxyz";
+  //  "# %         /          :;< >?                           [\]^ `                          {|}~
   //  ^--so these are encoded
         int i;
         // Encode control chars and space
@@ -724,11 +784,9 @@ public class Util {
      * Encode a single path component for use in an HTTP URL.
      * Escapes all non-ASCII, general unsafe (space and "#%<>[\]^`{|}~)
      * and HTTP special characters (/;:?) as specified in RFC1738.
-     * Also escapes & for convenience.
-     * (so alphanumeric and !@$*()-_=+',. are not encoded)
+     * (so alphanumeric and !@$&*()-_=+',. are not encoded)
      * Note that slash(/) is encoded, so the given string should be a
      * single path component used in constructing a URL.
-     * Do not use this method for encoding query parameters.
      * Method name inspired by PHP's rawurlencode.
      */
     public static String rawEncode(String s) {
@@ -973,54 +1031,78 @@ public class Util {
     }
 
     /**
-     * Creates a symlink to baseDir+targetPath at baseDir+symlinkPath.
+     * Creates a symlink to targetPath at baseDir+symlinkPath.
      * <p>
      * If there's a prior symlink at baseDir+symlinkPath, it will be overwritten.
      *
      * @param baseDir
      *      Base directory to resolve the 'symlinkPath' parameter.
      * @param targetPath
-     *      The file that the symlink should point to.
+     *      The file that the symlink should point to. Usually relative to the directory of the symlink but may instead be an absolute path.
      * @param symlinkPath
-     *      Where to create a symlink in.
+     *      Where to create a symlink in (relative to {@code baseDir})
      */
     public static void createSymlink(File baseDir, String targetPath, String symlinkPath, TaskListener listener) throws InterruptedException {
-        if(Functions.isWindows() || NO_SYMLINK)   return;
-
         try {
-            String errmsg = "";
-            // if a file or a directory exists here, delete it first.
-            // try simple delete first (whether exists() or not, as it may be symlink pointing
-            // to non-existent target), but fallback to "rm -rf" to delete non-empty dir.
-            File symlinkFile = new File(baseDir, symlinkPath);
-            if (!symlinkFile.delete() && symlinkFile.exists())
-                // ignore a failure.
-                new LocalProc(new String[]{"rm","-rf", symlinkPath},new String[0],listener.getLogger(), baseDir).join();
+            if (createSymlinkJava7(baseDir, targetPath, symlinkPath)) {
+                return;
+            }
+            if (NO_SYMLINK) {
+                return;
+            }
 
-            Integer r=null;
-            if (!SYMLINK_ESCAPEHATCH) {
+            File symlinkFile = new File(baseDir, symlinkPath);
+            if (Functions.isWindows()) {
+                if (symlinkFile.exists()) {
+                    symlinkFile.delete();
+                }
+                File dst = new File(symlinkFile,"..\\"+targetPath);
                 try {
-                    r = LIBC.symlink(targetPath,symlinkFile.getAbsolutePath());
-                    if (r!=0) {
-                        r = Native.getLastError();
-                        errmsg = LIBC.strerror(r);
+                    Kernel32Utils.createSymbolicLink(symlinkFile,targetPath,dst.isDirectory());
+                } catch (WinIOException e) {
+                    if (e.getErrorCode()==1314) {/* ERROR_PRIVILEGE_NOT_HELD */
+                        warnWindowsSymlink();
+                        return;
                     }
-                } catch (LinkageError e) {
-                    // if JNA is unavailable, fall back.
-                    // we still prefer to try JNA first as PosixAPI supports even smaller platforms.
-                    if (PosixAPI.supportsNative()) {
-                        r = PosixAPI.get().symlink(targetPath,symlinkFile.getAbsolutePath());
+                    throw e;
+                } catch (UnsatisfiedLinkError e) {
+                    // not available on this Windows
+                    return;
+                }
+            } else {
+                String errmsg = "";
+                // if a file or a directory exists here, delete it first.
+                // try simple delete first (whether exists() or not, as it may be symlink pointing
+                // to non-existent target), but fallback to "rm -rf" to delete non-empty dir.
+                if (!symlinkFile.delete() && symlinkFile.exists())
+                    // ignore a failure.
+                    new LocalProc(new String[]{"rm","-rf", symlinkPath},new String[0],listener.getLogger(), baseDir).join();
+
+                Integer r=null;
+                if (!SYMLINK_ESCAPEHATCH) {
+                    try {
+                        r = LIBC.symlink(targetPath,symlinkFile.getAbsolutePath());
+                        if (r!=0) {
+                            r = Native.getLastError();
+                            errmsg = LIBC.strerror(r);
+                        }
+                    } catch (LinkageError e) {
+                        // if JNA is unavailable, fall back.
+                        // we still prefer to try JNA first as PosixAPI supports even smaller platforms.
+                        if (PosixAPI.supportsNative()) {
+                            r = PosixAPI.get().symlink(targetPath,symlinkFile.getAbsolutePath());
+                        }
                     }
                 }
+                if (r==null) {
+                    // if all else fail, fall back to the most expensive approach of forking a process
+                    r = new LocalProc(new String[]{
+                        "ln","-s", targetPath, symlinkPath},
+                        new String[0],listener.getLogger(), baseDir).join();
+                }
+                if (r!=0)
+                    listener.getLogger().println(String.format("ln -s %s %s failed: %d %s",targetPath, symlinkFile, r, errmsg));
             }
-            if (r==null) {
-                // if all else fail, fall back to the most expensive approach of forking a process
-                r = new LocalProc(new String[]{
-                    "ln","-s", targetPath, symlinkPath},
-                    new String[0],listener.getLogger(), baseDir).join();
-            }
-            if (r!=0)
-                listener.getLogger().println(String.format("ln -s %s %s failed: %d %s",targetPath, symlinkFile, r, errmsg));
         } catch (IOException e) {
             PrintStream log = listener.getLogger();
             log.printf("ln %s %s failed%n",targetPath, new File(baseDir, symlinkPath));
@@ -1029,6 +1111,43 @@ public class Util {
         }
     }
 
+    private static boolean createSymlinkJava7(File baseDir, String targetPath, String symlinkPath) throws IOException {
+        try {
+            Object path = File.class.getMethod("toPath").invoke(new File(baseDir, symlinkPath));
+            Object target = Class.forName("java.nio.file.Paths").getMethod("get", String.class, String[].class).invoke(null, targetPath, new String[0]);
+            Class<?> filesC = Class.forName("java.nio.file.Files");
+            Class<?> pathC = Class.forName("java.nio.file.Path");
+            filesC.getMethod("deleteIfExists", pathC).invoke(null, path);
+            Object noAttrs = Array.newInstance(Class.forName("java.nio.file.attribute.FileAttribute"), 0);
+            filesC.getMethod("createSymbolicLink", pathC, pathC, noAttrs.getClass()).invoke(null, path, target, noAttrs);
+            return true;
+        } catch (NoSuchMethodException x) {
+            return false; // fine, Java 5/6
+        } catch (InvocationTargetException x) {
+            Throwable x2 = x.getCause();
+            if (x2 instanceof UnsupportedOperationException) {
+                return true; // no symlinks on this platform
+            }
+            if (Functions.isWindows() && String.valueOf(x2).contains("java.nio.file.FileSystemException")) {
+                warnWindowsSymlink();
+                return true;
+            }
+            if (x2 instanceof IOException) {
+                throw (IOException) x2;
+            }
+            throw (IOException) new IOException(x.toString()).initCause(x);
+        } catch (Exception x) {
+            throw (IOException) new IOException(x.toString()).initCause(x);
+        }
+    }
+
+    private static final AtomicBoolean warnedSymlinks = new AtomicBoolean();
+    private static void warnWindowsSymlink() {
+        if (warnedSymlinks.compareAndSet(false, true)) {
+            LOGGER.warning("Symbolic links enabled on this platform but disabled for this user; run as administrator or use Local Security Policy > Security Settings > Local Policies > User Rights Assignment > Create symbolic links");
+        }
+    }
+
     /**
      * @deprecated as of 1.456
      *      Use {@link #resolveSymlink(File)}
@@ -1037,15 +1156,58 @@ public class Util {
         return resolveSymlink(link);
     }
 
+    /**
+     * Resolves a symlink to the {@link File} that points to.
+     *
+     * @return null
+     *      if the specified file is not a symlink.
+     */
+    public static File resolveSymlinkToFile(File link) throws InterruptedException, IOException {
+        String target = resolveSymlink(link);
+        if (target==null)   return null;
+
+        File f = new File(target);
+        if (f.isAbsolute()) return f;   // absolute symlink
+        return new File(link.getParentFile(),target);   // relative symlink
+    }
+
     /**
      * Resolves symlink, if the given file is a symlink. Otherwise return null.
      * <p>
      * If the resolution fails, report an error.
      *
-     * @param listener
-     *      If we rely on an external command to resolve symlink, this is it.
+     * @return
+     *      null if the given file is not a symlink.
+     *      If the symlink is absolute, the returned string is an absolute path.
+     *      If the symlink is relative, the returned string is that relative representation.
+     *      The relative path is meant to be resolved from the location of the symlink.
      */
     public static String resolveSymlink(File link) throws InterruptedException, IOException {
+        try { // Java 7
+            Object path = File.class.getMethod("toPath").invoke(link);
+            return Class.forName("java.nio.file.Files").getMethod("readSymbolicLink", Class.forName("java.nio.file.Path")).invoke(null, path).toString();
+        } catch (NoSuchMethodException x) {
+            // fine, Java 5/6; fall through
+        } catch (InvocationTargetException x) {
+            Throwable x2 = x.getCause();
+            if (x2 instanceof UnsupportedOperationException) {
+                return null; // no symlinks on this platform
+            }
+            try {
+                if (Class.forName("java.nio.file.NotLinkException").isInstance(x2)) {
+                    return null;
+                }
+            } catch (ClassNotFoundException x3) {
+                assert false : x3; // should be Java 7+ here
+            }
+            if (x2 instanceof IOException) {
+                throw (IOException) x2;
+            }
+            throw (IOException) new IOException(x.toString()).initCause(x);
+        } catch (Exception x) {
+            throw (IOException) new IOException(x.toString()).initCause(x);
+        }
+
         if(Functions.isWindows())     return null;
 
         String filename = link.getAbsolutePath();
@@ -1132,7 +1294,6 @@ public class Util {
      * are overridden in the given derived type.
      */
     public static boolean isOverridden(Class base, Class derived, String methodName, Class... types) {
-        // the rewriteHudsonWar method isn't overridden.
         try {
             return !base.getMethod(methodName, types).equals(
                     derived.getMethod(methodName,types));
