@@ -25,6 +25,7 @@ package hudson.cli;
 
 import groovy.lang.GroovyShell;
 import groovy.lang.Binding;
+import hudson.cli.util.ScriptLoader;
 import hudson.model.AbstractProject;
 import jenkins.model.Jenkins;
 import hudson.model.Item;
@@ -53,10 +54,10 @@ import java.net.MalformedURLException;
  * @author Kohsuke Kawaguchi
  */
 @Extension
-public class GroovyCommand extends CLICommand implements Serializable {
+public class GroovyCommand extends CLICommand {
     @Override
     public String getShortDescription() {
-        return "Executes the specified Groovy script";
+        return Messages.GroovyCommand_ShortDescription();
     }
 
     @Argument(metaVar="SCRIPT",usage="Script to be executed. File, URL or '=' to represent stdin.")
@@ -69,8 +70,8 @@ public class GroovyCommand extends CLICommand implements Serializable {
     public List<String> remaining = new ArrayList<String>();
 
     protected int run() throws Exception {
-        // this allows the caller to manipulate the JVM state, so require the admin privilege.
-        Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
+        // this allows the caller to manipulate the JVM state, so require the execute script privilege.
+        Jenkins.getInstance().checkPermission(Jenkins.RUN_SCRIPTS);
 
         Binding binding = new Binding();
         binding.setProperty("out",new PrintWriter(stdout,true));
@@ -103,26 +104,7 @@ public class GroovyCommand extends CLICommand implements Serializable {
         if (script.equals("="))
             return IOUtils.toString(stdin);
 
-        return channel.call(new Callable<String,IOException>() {
-            public String call() throws IOException {
-                File f = new File(script);
-                if(f.exists())
-                    return FileUtils.readFileToString(f);
-
-                URL url;
-                try {
-                    url = new URL(script);
-                } catch (MalformedURLException e) {
-                    throw new AbortException("Unable to find a script "+script);
-                }
-                InputStream s = url.openStream();
-                try {
-                    return IOUtils.toString(s);
-                } finally {
-                    s.close();
-                }
-            }
-        });
+        return checkChannel().call(new ScriptLoader(script));
     }
 }
 
