@@ -33,8 +33,10 @@ import hudson.model.listeners.SaveableListener;
 import hudson.node_monitors.NodeMonitor;
 import hudson.slaves.NodeDescriptor;
 import hudson.util.DescribableList;
+import hudson.util.FormApply;
 import hudson.util.FormValidation;
 import jenkins.model.Jenkins;
+import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.QueryParameter;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
@@ -47,11 +49,13 @@ import static javax.servlet.http.HttpServletResponse.SC_BAD_REQUEST;
 import java.io.File;
 import java.io.IOException;
 import java.util.AbstractList;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import net.sf.json.JSONObject;
 
 /**
  * Serves as the top of {@link Computer}s in the URL hierarchy.
@@ -167,7 +171,7 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
     public int getIdleExecutors() {
         int r=0;
         for (Computer c : get_all())
-            if(c.isOnline() || c.isConnecting())
+            if((c.isOnline() || c.isConnecting()) && c.isAcceptingTasks())
                 r += c.countIdle();
         return r;
     }
@@ -231,6 +235,10 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
             String xml = Jenkins.XSTREAM.toXML(src);
             Node result = (Node) Jenkins.XSTREAM.fromXML(xml);
             result.setNodeName(name);
+            if(result instanceof Slave){ //change userId too
+                User user = User.current();
+                ((Slave)result).setUserId(user==null ? "anonymous" : user.getId());
+             }
             result.holdOffLaunchUntilSave = true;
 
             app.addNode(result);
@@ -257,9 +265,13 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
                                            @QueryParameter String type ) throws IOException, ServletException, FormException {
         final Jenkins app = Jenkins.getInstance();
         app.checkPermission(Computer.CREATE);
-        checkName(name);
+        String fixedName = Util.fixEmptyAndTrim(name);
+        checkName(fixedName);
 
-        Node result = NodeDescriptor.all().find(type).newInstance(req, req.getSubmittedForm());
+        JSONObject formData = req.getSubmittedForm();
+        formData.put("name", fixedName);
+        
+        Node result = NodeDescriptor.all().find(type).newInstance(req, formData);
         app.addNode(result);
 
         // take the user back to the slave list top page
@@ -305,7 +317,7 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
      * Accepts submission from the configuration page.
      */
     @RequirePOST
-    public synchronized void doConfigSubmit( StaplerRequest req, StaplerResponse rsp ) throws IOException, ServletException, FormException {
+    public synchronized HttpResponse doConfigSubmit( StaplerRequest req) throws IOException, ServletException, FormException {
         BulkChange bc = new BulkChange(MONITORS_OWNER);
         try {
             Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
@@ -324,7 +336,7 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
                 nm.triggerUpdate();
             }
 
-            rsp.sendRedirect2(".");
+            return FormApply.success(".");
         } finally {
             bc.commit();
         }
@@ -384,7 +396,16 @@ public final class ComputerSet extends AbstractModelObject implements Describabl
             if(xf.exists()) {
                 DescribableList<NodeMonitor,Descriptor<NodeMonitor>> persisted =
                         (DescribableList<NodeMonitor,Descriptor<NodeMonitor>>) xf.read();
-                r.replaceBy(persisted.toList());
+                List<NodeMonitor> sanitized = new ArrayList<NodeMonitor>();
+                for (NodeMonitor nm : persisted) {
+                    try {
+                        nm.getDescriptor();
+                        sanitized.add(nm);
+                    } catch (Throwable e) {
+                        // the descriptor didn't load? see JENKINS-15869
+                    }
+                }
+                r.replaceBy(sanitized);
             }
 
             // if we have any new monitors, let's add them
