@@ -31,7 +31,6 @@ import hudson.BulkChange;
 import hudson.Util;
 import hudson.model.listeners.SaveableListener;
 import hudson.util.FormApply;
-import hudson.util.QuotedStringTokenizer;
 import hudson.util.ReflectionUtils;
 import hudson.util.ReflectionUtils.Parameter;
 import hudson.views.ListViewColumn;
@@ -73,6 +72,7 @@ import java.lang.reflect.Field;
 import java.lang.reflect.ParameterizedType;
 import java.beans.Introspector;
 import javax.annotation.CheckForNull;
+import javax.annotation.Nonnull;
 
 /**
  * Metadata about a configurable instance.
@@ -194,6 +194,9 @@ public abstract class Descriptor<T extends Describable<T>> implements Saveable {
 
         public Descriptor getItemTypeDescriptorOrDie() {
             Class it = getItemType();
+            if (it == null) {
+                throw new AssertionError(clazz + " is not an array/collection type in " + displayName + ". See https://wiki.jenkins-ci.org/display/JENKINS/My+class+is+missing+descriptor");
+            }
             Descriptor d = Jenkins.getInstance().getDescriptor(it);
             if (d==null)
                 throw new AssertionError(it +" is missing its descriptor in "+displayName+". See https://wiki.jenkins-ci.org/display/JENKINS/My+class+is+missing+descriptor");
@@ -201,11 +204,18 @@ public abstract class Descriptor<T extends Describable<T>> implements Saveable {
         }
 
         /**
-         * Returns all the descriptors that produce types assignable to the item type.
+         * Returns all the descriptors that produce types assignable to the property type.
          */
         public List<? extends Descriptor> getApplicableDescriptors() {
             return Jenkins.getInstance().getDescriptorList(clazz);
         }
+
+        /**
+         * Returns all the descriptors that produce types assignable to the item type for a collection property.
+         */
+        public List<? extends Descriptor> getApplicableItemDescriptors() {
+            return Jenkins.getInstance().getDescriptorList(getItemType());
+        }
     }
 
     /**
@@ -213,7 +223,7 @@ public abstract class Descriptor<T extends Describable<T>> implements Saveable {
      *
      * @see #getHelpFile(String) 
      */
-    private final Map<String,String> helpRedirect = new HashMap<String, String>();
+    private transient final Map<String,String> helpRedirect = new HashMap<String, String>();
 
     /**
      *
@@ -461,11 +471,27 @@ public abstract class Descriptor<T extends Describable<T>> implements Saveable {
     /**
      * Used by Jelly to abstract away the handlign of global.jelly vs config.jelly databinding difference.
      */
-    public PropertyType getPropertyType(Object instance, String field) {
+    public @CheckForNull PropertyType getPropertyType(@Nonnull Object instance, @Nonnull String field) {
         // in global.jelly, instance==descriptor
         return instance==this ? getGlobalPropertyType(field) : getPropertyType(field);
     }
 
+    /**
+     * Akin to {@link #getPropertyType(Object,String) but never returns null.
+     * @throws AssertionError in case the field cannot be found
+     * @since 1.492
+     */
+    public @Nonnull PropertyType getPropertyTypeOrDie(@Nonnull Object instance, @Nonnull String field) {
+        PropertyType propertyType = getPropertyType(instance, field);
+        if (propertyType != null) {
+            return propertyType;
+        } else if (instance == this) {
+            throw new AssertionError(getClass().getName() + " has no property " + field);
+        } else {
+            throw new AssertionError(clazz.getName() + " has no property " + field);
+        }
+    }
+
     /**
      * Obtains the property type of the given field of {@link #clazz}
      */
@@ -783,7 +809,7 @@ public abstract class Descriptor<T extends Describable<T>> implements Saveable {
         }
     }
 
-    private XmlFile getConfigFile() {
+    protected XmlFile getConfigFile() {
         return new XmlFile(new File(Jenkins.getInstance().getRootDir(),getId()+".xml"));
     }
 
