@@ -77,17 +77,20 @@ public class FileParameterValue extends ParameterValue {
         this(name, new FileItemImpl(file), originalFileName);
     }
 
-    private FileParameterValue(String name, FileItem file, String originalFileName) {
+    protected FileParameterValue(String name, FileItem file, String originalFileName) {
         super(name);
         this.file = file;
         this.originalFileName = originalFileName;
     }
 
     // post initialization hook
-    /*package*/ void setLocation(String location) {
+    protected void setLocation(String location) {
         this.location = location;
     }
 
+    public String getLocation() {
+        return location;
+    }
 
     /**
      * Exposes the originalFileName as an environment variable.
@@ -117,6 +120,10 @@ public class FileParameterValue extends ParameterValue {
         return originalFileName;
     }
 
+    public FileItem getFile() {
+        return file;
+    }
+
     @Override
     public BuildWrapper createBuildWrapper(AbstractBuild<?,?> build) {
         return new BuildWrapper() {
@@ -164,7 +171,7 @@ public class FileParameterValue extends ParameterValue {
 	}
 
     @Override
-    public String getShortDescription() {
+    public String toString() {
     	return "(FileParameterValue) " + getName() + "='" + originalFileName + "'";
     }
 
