@@ -26,6 +26,7 @@ package hudson.model;
 import com.google.common.base.Function;
 import com.google.common.collect.Collections2;
 import com.infradna.tool.bridge_method_injector.WithBridgeMethods;
+import hudson.BulkChange;
 import hudson.EnvVars;
 import hudson.Extension;
 import hudson.ExtensionPoint;
@@ -52,6 +53,7 @@ import hudson.util.DataSetBuilder;
 import hudson.util.DescribableList;
 import hudson.util.FormApply;
 import hudson.util.Graph;
+import hudson.util.ProcessTree;
 import hudson.util.RunList;
 import hudson.util.ShiftedCategoryAxis;
 import hudson.util.StackedAreaRenderer2;
@@ -59,8 +61,10 @@ import hudson.util.TextFile;
 import hudson.widgets.HistoryWidget;
 import hudson.widgets.HistoryWidget.Adapter;
 import hudson.widgets.Widget;
+import jenkins.model.BuildDiscarder;
 import jenkins.model.Jenkins;
 import jenkins.model.ProjectNamingStrategy;
+import jenkins.scm.SCMCheckoutStrategy;
 import jenkins.security.HexStringConfidentialKey;
 import jenkins.util.io.OnMaster;
 import net.sf.json.JSONException;
@@ -123,7 +127,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      */
     private transient volatile boolean holdOffBuildUntilSave;
 
-    private volatile LogRotator logRotator;
+    private volatile BuildDiscarder logRotator;
 
     /**
      * Not all plugins are good at calculating their health report quickly.
@@ -188,7 +192,6 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         } else {
             // From the old Hudson, or doCreateItem. Create this file now.
             saveNextBuildNumber();
-            save(); // and delete it from the config.xml
         }
 
         if (properties == null) // didn't exist < 1.72
@@ -302,6 +305,47 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         return nextBuildNumber;
     }
 
+    /**
+     * Builds up the environment variable map that's sufficient to identify a process
+     * as ours. This is used to kill run-away processes via {@link ProcessTree#killAll(Map)}.
+     */
+    public EnvVars getCharacteristicEnvVars() {
+        EnvVars env = new EnvVars();
+        env.put("JENKINS_SERVER_COOKIE",SERVER_COOKIE.get());
+        env.put("HUDSON_SERVER_COOKIE",SERVER_COOKIE.get()); // Legacy compatibility
+        env.put("JOB_NAME",getFullName());
+        return env;
+    }
+
+    /**
+     * Creates an environment variable override for launching processes for this project.
+     *
+     * <p>
+     * This is for process launching outside the build execution (such as polling, tagging, deployment, etc.)
+     * that happens in a context of a specific job.
+     *
+     * @param node
+     *      Node to eventually run a process on. The implementation must cope with this parameter being null
+     *      (in which case none of the node specific properties would be reflected in the resulting override.)
+     */
+    public EnvVars getEnvironment(Node node, TaskListener listener) throws IOException, InterruptedException {
+        EnvVars env;
+
+        if (node!=null)
+            env = node.toComputer().buildEnvironment(listener);
+        else
+            env = new EnvVars();
+
+        env.putAll(getCharacteristicEnvVars());
+
+        // servlet container may have set CLASSPATH in its launch script,
+        // so don't let that inherit to the new child process.
+        // see http://www.nabble.com/Run-Job-with-JDK-1.4.2-tf4468601.html
+        env.put("CLASSPATH","");
+
+        return env;
+    }
+
     /**
      * Programatically updates the next build number.
      * 
@@ -321,23 +365,45 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
     }
 
     /**
-     * Returns the log rotator for this job, or null if none.
+     * Returns the configured build discarder for this job, or null if none.
      */
-    public LogRotator getLogRotator() {
+    public BuildDiscarder getBuildDiscarder() {
         return logRotator;
     }
 
-    public void setLogRotator(LogRotator logRotator) {
-        this.logRotator = logRotator;
+    public void setBuildDiscarder(BuildDiscarder bd) throws IOException {
+        this.logRotator = bd;
+        save();
+    }
+
+    /**
+     * Left for backward compatibility. Returns non-null if and only
+     * if {@link LogRotator} is configured as {@link BuildDiscarder}.
+     *
+     * @deprecated as of 1.503
+     *      Use {@link #getBuildDiscarder()}.
+     */
+    public LogRotator getLogRotator() {
+        if (logRotator instanceof LogRotator)
+            return (LogRotator) logRotator;
+        return null;
+    }
+
+    /**
+     * @deprecated as of 1.503
+     *      Use {@link #setBuildDiscarder(BuildDiscarder)}
+     */
+    public void setLogRotator(LogRotator logRotator) throws IOException {
+        setBuildDiscarder(logRotator);
     }
 
     /**
      * Perform log rotation.
      */
     public void logRotate() throws IOException, InterruptedException {
-        LogRotator lr = getLogRotator();
-        if (lr != null)
-            lr.perform(this);
+        BuildDiscarder bd = getBuildDiscarder();
+        if (bd != null)
+            bd.perform(this);
     }
 
     /**
@@ -455,6 +521,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
 
     /**
      * Overrides from job properties.
+     * @see JobProperty#getJobOverrides
      */
     public Collection<?> getOverrides() {
         List<Object> r = new ArrayList<Object>();
@@ -506,7 +573,19 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      */
     @Override
     public void renameTo(String newName) throws IOException {
+        File oldBuildDir = getBuildDir();
         super.renameTo(newName);
+        File newBuildDir = getBuildDir();
+        if (oldBuildDir.isDirectory() && !newBuildDir.isDirectory()) {
+            if (!oldBuildDir.renameTo(newBuildDir)) {
+                throw new IOException("failed to rename " + oldBuildDir + " to " + newBuildDir);
+            }
+        }
+    }
+
+    @Override public synchronized void delete() throws IOException, InterruptedException {
+        super.delete();
+        Util.deleteRecursive(getBuildDir());
     }
 
     /**
@@ -520,12 +599,22 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      * 
      * @return never null. The first entry is the latest build.
      */
-    @Exported
+    @Exported(name="allBuilds",visibility=-2)
     @WithBridgeMethods(List.class)
     public RunList<RunT> getBuilds() {
         return RunList.fromRuns(_getRuns().values());
     }
 
+    /**
+     * Gets the read-only view of the recent builds.
+     *
+     * @since 1.485
+     */
+    @Exported(name="builds")
+    public RunList<RunT> getNewBuilds() {
+        return getBuilds().limit(100);
+    }
+
     /**
      * Obtains all the {@link Run}s whose build numbers matches the given {@link RangeSet}.
      */
@@ -602,7 +691,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      * This is useful when you'd like to fetch a build but the exact build might
      * be already gone (deleted, rotated, etc.)
      */
-    public final RunT getNearestBuild(int n) {
+    public RunT getNearestBuild(int n) {
         SortedMap<Integer, ? extends RunT> m = _getRuns().headMap(n - 1); // the map should
                                                                           // include n, so n-1
         if (m.isEmpty())
@@ -616,7 +705,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      * This is useful when you'd like to fetch a build but the exact build might
      * be already gone (deleted, rotated, etc.)
      */
-    public final RunT getNearestOldBuild(int n) {
+    public RunT getNearestOldBuild(int n) {
         SortedMap<Integer, ? extends RunT> m = _getRuns().tailMap(n);
         if (m.isEmpty())
             return null;
@@ -628,7 +717,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
             StaplerResponse rsp) {
         try {
             // try to interpret the token as build number
-            return _getRuns().get(Integer.valueOf(token));
+            return getBuildByNumber(Integer.valueOf(token));
         } catch (NumberFormatException e) {
             // try to map that to widgets
             for (Widget w : getWidgets()) {
@@ -655,14 +744,14 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
      * 
      * @see RunMap
      */
-    protected File getBuildDir() {
+    public File getBuildDir() {
         return Jenkins.getInstance().getBuildDirFor(this);
     }
 
     /**
      * Gets all the runs.
      * 
-     * The resulting map must be immutable (by employing copy-on-write
+     * The resulting map must be treated immutable (by employing copy-on-write
      * semantics.) The map is descending order, with newest builds at the top.
      */
     protected abstract SortedMap<Integer, ? extends RunT> _getRuns();
@@ -710,13 +799,7 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
     @Exported
     @QuickSilver
     public RunT getLastSuccessfulBuild() {
-        RunT r = getLastBuild();
-        // temporary hack till we figure out what's causing this bug
-        while (r != null
-                && (r.isBuilding() || r.getResult() == null || r.getResult()
-                        .isWorseThan(Result.UNSTABLE)))
-            r = r.getPreviousBuild();
-        return r;
+        return (RunT)Permalink.LAST_SUCCESSFUL_BUILD.resolve(this);
     }
 
     /**
@@ -977,8 +1060,8 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
 
             setDisplayName(json.optString("displayNameOrNull"));
 
-            if (req.getParameter("logrotate") != null)
-                logRotator = LogRotator.DESCRIPTOR.newInstance(req,json.getJSONObject("logrotate"));
+            if (json.optBoolean("logrotate"))
+                logRotator = req.bindJSON(BuildDiscarder.class, json.optJSONObject("buildDiscarder"));
             else
                 logRotator = null;
 
@@ -1262,5 +1345,5 @@ public abstract class Job<JobT extends Job<JobT, RunT>, RunT extends Run<JobT, R
         return new BuildTimelineWidget(getBuilds());
     }
 
-    /*package*/ final static HexStringConfidentialKey SERVER_COOKIE = new HexStringConfidentialKey(Job.class,"serverCookie",16);
+    private final static HexStringConfidentialKey SERVER_COOKIE = new HexStringConfidentialKey(Job.class,"serverCookie",16);
 }
