@@ -27,9 +27,11 @@ import hudson.ExtensionPoint;
 import hudson.ExtensionListView;
 import hudson.Extension;
 import hudson.ExtensionList;
+import hudson.security.Permission;
 import jenkins.model.Jenkins;
 
 import java.util.List;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 /**
  * Extension point to add icon to <tt>http://server/hudson/manage</tt> page.
@@ -79,6 +81,17 @@ public abstract class ManagementLink implements ExtensionPoint, Action {
      */
     public abstract String getUrlName();
 
+    /**
+     * Allows implementations to request that this link show a confirmation dialog, and use POST if confirmed.
+     * Suitable for links which perform an action rather than simply displaying a page.
+     * @return true if this link takes an action
+     * @see RequirePOST
+     * @since 1.512
+     */
+    public boolean getRequiresConfirmation() {
+        return false;
+    }
+
     /**
      * All registered instances.
      * @deprecated as of 1.286
@@ -92,4 +105,11 @@ public abstract class ManagementLink implements ExtensionPoint, Action {
     public static ExtensionList<ManagementLink> all() {
         return Jenkins.getInstance().getExtensionList(ManagementLink.class);
     }
+
+    /**
+     * @return permission required for user to access this management link, in addition to {@link Jenkins.ADMINISTER}
+     */
+    public Permission getRequiredPermission() {
+        return null;
+    }
 }
