@@ -41,6 +41,7 @@ import hudson.model.UpdateSite.Plugin;
 import hudson.model.listeners.SaveableListener;
 import hudson.security.ACL;
 import hudson.util.DaemonThreadFactory;
+import hudson.util.FormValidation;
 import hudson.util.HttpResponses;
 import hudson.util.IOException2;
 import hudson.util.PersistedList;
@@ -76,6 +77,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeSet;
 import java.util.Vector;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
@@ -111,9 +113,16 @@ import org.kohsuke.stapler.interceptor.RequirePOST;
 public class UpdateCenter extends AbstractModelObject implements Saveable, OnMaster {
 	
     private static final String UPDATE_CENTER_URL = System.getProperty(UpdateCenter.class.getName()+".updateCenterUrl","http://updates.jenkins-ci.org/");
+
+    /**
+     * {@linkplain UpdateSite#getId() ID} of the default update site.
+     * @since 1.483
+     */
+    public static final String ID_DEFAULT = "default";
 	
     /**
      * {@link ExecutorService} that performs installation.
+     * @since 1.501
      */
     private final ExecutorService installerService = Executors.newSingleThreadExecutor(
         new DaemonThreadFactory(new ThreadFactory() {
@@ -124,6 +133,18 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
             }
         }));
 
+    /**
+     * An {@link ExecutorService} for updating UpdateSites.
+     */
+    protected final ExecutorService updateService = Executors.newCachedThreadPool(
+        new DaemonThreadFactory(new ThreadFactory() {
+            public Thread newThread(Runnable r) {
+                Thread t = new Thread(r);
+                t.setName("Update site data downloader");
+                return t;
+            }
+        }));
+        
     /**
      * List of created {@link UpdateCenterJob}s. Access needs to be synchronized.
      */
@@ -145,6 +166,8 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      */
     private UpdateCenterConfiguration config;
 
+    private boolean requiresRestart;
+
     public UpdateCenter() {
         configure(new UpdateCenterConfiguration());
     }
@@ -180,6 +203,21 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
         }
     }
 
+    /**
+     * Gets a job by its ID.
+     *
+     * Primarily to make {@link UpdateCenterJob} bound to URL.
+     */
+    public UpdateCenterJob getJob(int id) {
+        synchronized (jobs) {
+            for (UpdateCenterJob job : jobs) {
+                if (job.id==id)
+                    return job;
+            }
+        }
+        return null;
+    }
+
     /**
      * Returns latest install/upgrade job for the given plugin.
      * @return InstallationJob or null if not found
@@ -228,11 +266,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
         return sites.toList();
     }
 
+    /**
+     * Alias for {@link #getById}.
+     */
     public UpdateSite getSite(String id) {
-        for (UpdateSite site : sites)
-            if (site.getId().equals(id))
-                return site;
-        return null;
+        return getById(id);
     }
 
     /**
@@ -240,13 +278,15 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      * Will be the newest of all {@link UpdateSite}s.
      */
     public String getLastUpdatedString() {
-        long newestTs = -1;
+        long newestTs = 0;
         for (UpdateSite s : sites) {
             if (s.getDataTimestamp()>newestTs) {
                 newestTs = s.getDataTimestamp();
             }
         }
-        if(newestTs<0)     return "N/A";
+        if (newestTs == 0) {
+            return Messages.UpdateCenter_n_a();
+        }
         return Util.getPastTimeString(System.currentTimeMillis()-newestTs);
     }
 
@@ -361,10 +401,27 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
         }
         response.sendRedirect2(".");
     }
-    
+
     /**
-     * Checks if restart is scheduled
-     * 
+     * If any of the executed {@link UpdateCenterJob}s requires a restart
+     * to take effect, this method returns true.
+     *
+     * <p>
+     * This doesn't necessarily mean the user has scheduled or initiated
+     * the restart operation.
+     *
+     * @see #isRestartScheduled()
+     */
+    @Exported
+    public boolean isRestartRequiredForCompletion() {
+        return requiresRestart;
+    }
+
+    /**
+     * Checks if the restart operation is scheduled
+     * (which means in near future Jenkins will restart by itself)
+     *
+     * @see #isRestartRequiredForCompletion()
      */
     public boolean isRestartScheduled() {
         for (UpdateCenterJob job : getJobs()) {
@@ -464,7 +521,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      * Loads the data from the disk into this object.
      */
     public synchronized void load() throws IOException {
-        UpdateSite defaultSite = new UpdateSite("default", config.getUpdateCenterUrl() + "update-center.json");
+        UpdateSite defaultSite = new UpdateSite(ID_DEFAULT, config.getUpdateCenterUrl() + "update-center.json");
         XmlFile file = getConfigFile();
         if(file.exists()) {
             try {
@@ -562,6 +619,32 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
 
         return new ArrayList<Plugin>(pluginMap.values());
     }
+    
+    /**
+     * Ensure that all UpdateSites are up to date, without requiring a user to
+     * browse to the instance.
+     * 
+     * @return a list of {@link FormValidation} for each updated Update Site
+     * @throws ExecutionException 
+     * @throws InterruptedException 
+     * @since 1.501
+     * 
+     */
+    public List<FormValidation> updateAllSites() throws InterruptedException, ExecutionException {
+        List <Future<FormValidation>> futures = new ArrayList<Future<FormValidation>>();
+        for (UpdateSite site : getSites()) {
+            Future<FormValidation> future = site.updateDirectly(true);
+            if (future != null) {
+                futures.add(future);
+            }
+        }
+        
+        List<FormValidation> results = new ArrayList<FormValidation>(); 
+        for (Future<FormValidation> f : futures) {
+            results.add(f.get());
+        }
+        return results;
+    }
 
 
     /**
@@ -792,6 +875,14 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      */
     @ExportedBean
     public abstract class UpdateCenterJob implements Runnable {
+        /**
+         * Unique ID that identifies this job.
+         *
+         * @see UpdateCenter#getJob(int)
+         */
+        @Exported
+        public final int id = iota.incrementAndGet();
+
         /**
          * Which {@link UpdateSite} does this belong to?
          */
@@ -835,6 +926,10 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
         }
 
         @Exported
+        public String getErrorMessage() {
+            return error != null ? error.getMessage() : null;
+        }
+        
         public Throwable getError() {
             return error;
         }
@@ -844,12 +939,6 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      * Restarts jenkins.
      */
     public class RestartJenkinsJob extends UpdateCenterJob {
-        /**
-         * Unique ID that identifies this job.
-         */
-        @Exported
-        public final int id = iota.incrementAndGet();
-               
          /**
          * Immutable state of this job.
          */
@@ -968,11 +1057,6 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
      * Base class for a job that downloads a file from the Jenkins project.
      */
     public abstract class DownloadJob extends UpdateCenterJob {
-        /**
-         * Unique ID that identifies this job.
-         */
-        @Exported
-        public final int id = iota.incrementAndGet();
         /**
          * Immutable object representing the current state of this job.
          */
@@ -1024,6 +1108,7 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
             } catch (InstallationStatus e) {
                 status = e;
                 if (status.isSuccess()) onSuccess();
+                requiresRestart |= status.requiresRestart();
             } catch (Throwable e) {
                 LOGGER.log(Level.SEVERE, "Failed to install "+getName(),e);
                 status = new Failure(e);
@@ -1073,6 +1158,13 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
             public final String getType() {
                 return getClass().getSimpleName();
             }
+
+            /**
+             * Indicates that a restart is needed to complete the tasks.
+             */
+            public boolean requiresRestart() {
+                return false;
+            }
         }
 
         /**
@@ -1105,6 +1197,11 @@ public class UpdateCenter extends AbstractModelObject implements Saveable, OnMas
             public String getMessage() {
                 return message.toString();
             }
+
+            @Override
+            public boolean requiresRestart() {
+                return true;
+            }
         }
 
         /**
