@@ -25,9 +25,9 @@
 
 package hudson.model;
 
-import com.trilead.ssh2.crypto.Base64;
 import hudson.PluginManager;
 import hudson.PluginWrapper;
+import hudson.ProxyConfiguration;
 import hudson.lifecycle.Lifecycle;
 import hudson.model.UpdateCenter.UpdateCenterJob;
 import hudson.util.FormValidation;
@@ -37,12 +37,9 @@ import hudson.util.IOUtils;
 import hudson.util.TextFile;
 import hudson.util.VersionNumber;
 import jenkins.model.Jenkins;
+import jenkins.util.JSONSignatureValidator;
 import net.sf.json.JSONException;
 import net.sf.json.JSONObject;
-import org.apache.commons.io.output.NullOutputStream;
-import org.apache.commons.io.output.TeeOutputStream;
-import org.jvnet.hudson.crypto.CertificateUtil;
-import org.jvnet.hudson.crypto.SignatureOutputStream;
 import org.kohsuke.stapler.DataBoundConstructor;
 import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.StaplerRequest;
@@ -50,28 +47,22 @@ import org.kohsuke.stapler.export.Exported;
 import org.kohsuke.stapler.export.ExportedBean;
 import org.kohsuke.stapler.interceptor.RequirePOST;
 
-import java.io.ByteArrayInputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
-import java.io.OutputStreamWriter;
-import java.security.DigestOutputStream;
+import java.io.InputStream;
+import java.net.URI;
+import java.net.URL;
+import java.net.URLConnection;
+import java.net.URLEncoder;
 import java.security.GeneralSecurityException;
-import java.security.MessageDigest;
-import java.security.Signature;
-import java.security.cert.CertificateExpiredException;
-import java.security.cert.CertificateFactory;
-import java.security.cert.CertificateNotYetValidException;
-import java.security.cert.TrustAnchor;
-import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.TreeMap;
+import java.util.concurrent.Callable;
 import java.util.concurrent.Future;
 import java.util.logging.Level;
 import java.util.logging.Logger;
@@ -94,18 +85,20 @@ import static hudson.util.TimeUnit2.*;
 public class UpdateSite {
     /**
      * What's the time stamp of data file?
+     * 0 means never.
      */
-    private transient long dataTimestamp = -1;
+    private transient volatile long dataTimestamp;
 
     /**
      * When was the last time we asked a browser to check the data for us?
+     * 0 means never.
      *
      * <p>
      * There's normally some delay between when we send HTML that includes the check code,
      * until we get the data back, so this variable is used to avoid asking too many browseres
      * all at once.
      */
-    private transient volatile long lastAttempt = -1;
+    private transient volatile long lastAttempt;
 
     /**
      * If the attempt to fetch data fails, we progressively use longer time out before retrying,
@@ -121,7 +114,7 @@ public class UpdateSite {
     /**
      * Latest data as read from the data file.
      */
-    private Data data;
+    private transient Data data;
 
     /**
      * ID string for this update source.
@@ -138,14 +131,6 @@ public class UpdateSite {
         this.url = url;
     }
 
-    /**
-     * When read back from XML, initialize them back to -1.
-     */
-    private Object readResolve() {
-        dataTimestamp = lastAttempt = -1;
-        return this;
-    }
-
     /**
      * Get ID string.
      */
@@ -156,25 +141,77 @@ public class UpdateSite {
 
     @Exported
     public long getDataTimestamp() {
+        assert dataTimestamp >= 0;
         return dataTimestamp;
     }
 
+    /**
+     * Update the data file from the given URL if the file
+     * does not exist, or is otherwise due for update.
+     * Accepted formats are JSONP or HTML with {@code postMessage}, not raw JSON.
+     * @param signatureCheck whether to enforce the signature (may be off only for testing!)
+     * @return null if no updates are necessary, or the future result
+     * @since 1.502
+     */
+    public Future<FormValidation> updateDirectly(final boolean signatureCheck) {
+        if (! getDataFile().exists() || isDue()) {
+            return Jenkins.getInstance().getUpdateCenter().updateService.submit(new Callable<FormValidation>() {
+                
+                public FormValidation call() throws Exception {
+                    URL src = new URL(getUrl() + "?id=" + URLEncoder.encode(getId(),"UTF-8") 
+                            + "&version="+URLEncoder.encode(Jenkins.VERSION, "UTF-8"));
+                    URLConnection conn = ProxyConfiguration.open(src);
+                    InputStream is = conn.getInputStream();
+                    try {
+                        String uncleanJson = IOUtils.toString(is,"UTF-8");
+                        int jsonStart = uncleanJson.indexOf("{\"");
+                        if (jsonStart >= 0) {
+                            uncleanJson = uncleanJson.substring(jsonStart);
+                            int end = uncleanJson.lastIndexOf('}');
+                            if (end>0)
+                                uncleanJson = uncleanJson.substring(0,end+1);
+                            return updateData(uncleanJson, signatureCheck);
+                        } else {
+                            throw new IOException("Could not find json in content of " +
+                            		"update center from url: "+src.toExternalForm());
+                        }
+                    } finally {
+                        if (is != null)
+                            is.close();
+                    }
+                }
+            });
+        }
+            return null;
+    }
+    
     /**
      * This is the endpoint that receives the update center data file from the browser.
      */
     public FormValidation doPostBack(StaplerRequest req) throws IOException, GeneralSecurityException {
+        return updateData(IOUtils.toString(req.getInputStream(),"UTF-8"), true);
+    }
+
+    private FormValidation updateData(String json, boolean signatureCheck)
+            throws IOException {
+
         dataTimestamp = System.currentTimeMillis();
-        String json = IOUtils.toString(req.getInputStream(),"UTF-8");
+
         JSONObject o = JSONObject.fromObject(json);
 
-        int v = o.getInt("updateCenterVersion");
-        if(v !=1)
-            throw new IllegalArgumentException("Unrecognized update center version: "+v);
+        try {
+            int v = o.getInt("updateCenterVersion");
+            if (v != 1) {
+                throw new IllegalArgumentException("Unrecognized update center version: " + v);
+            }
+        } catch (JSONException x) {
+            throw new IllegalArgumentException("Could not find (numeric) updateCenterVersion in " + json, x);
+        }
 
         if (signatureCheck) {
             FormValidation e = verifySignature(o);
             if (e.kind!=Kind.OK) {
-                LOGGER.severe(e.renderHtml());
+                LOGGER.severe(e.toString());
                 return e;
             }
         }
@@ -193,100 +230,7 @@ public class UpdateSite {
      * Verifies the signature in the update center data file.
      */
     private FormValidation verifySignature(JSONObject o) throws IOException {
-        try {
-            FormValidation warning = null;
-
-            JSONObject signature = o.getJSONObject("signature");
-            if (signature.isNullObject()) {
-                return FormValidation.error("No signature block found in update center '"+id+"'");
-            }
-            o.remove("signature");
-
-            List<X509Certificate> certs = new ArrayList<X509Certificate>();
-            {// load and verify certificates
-                CertificateFactory cf = CertificateFactory.getInstance("X509");
-                for (Object cert : signature.getJSONArray("certificates")) {
-                    X509Certificate c = (X509Certificate) cf.generateCertificate(new ByteArrayInputStream(Base64.decode(cert.toString().toCharArray())));
-                    try {
-                        c.checkValidity();
-                    } catch (CertificateExpiredException e) { // even if the certificate isn't valid yet, we'll proceed it anyway
-                        warning = FormValidation.warning(e,String.format("Certificate %s has expired in update center '%s'",cert.toString(),id));
-                    } catch (CertificateNotYetValidException e) {
-                        warning = FormValidation.warning(e,String.format("Certificate %s is not yet valid in update center '%s'",cert.toString(),id));
-                    }
-                    certs.add(c);
-                }
-
-                // if we trust default root CAs, we end up trusting anyone who has a valid certificate,
-                // which isn't useful at all
-                Set<TrustAnchor> anchors = new HashSet<TrustAnchor>(); // CertificateUtil.getDefaultRootCAs();
-                Jenkins j = Jenkins.getInstance();
-                for (String cert : (Set<String>) j.servletContext.getResourcePaths("/WEB-INF/update-center-rootCAs")) {
-                    if (cert.endsWith(".txt"))  continue;       // skip text files that are meant to be documentation
-                    anchors.add(new TrustAnchor((X509Certificate)cf.generateCertificate(j.servletContext.getResourceAsStream(cert)),null));
-                }
-                File[] cas = new File(j.root, "update-center-rootCAs").listFiles();
-                if (cas!=null) {
-                    for (File cert : cas) {
-                        if (cert.getName().endsWith(".txt"))  continue;       // skip text files that are meant to be documentation
-                        FileInputStream in = new FileInputStream(cert);
-                        try {
-                            anchors.add(new TrustAnchor((X509Certificate)cf.generateCertificate(in),null));
-                        } finally {
-                            in.close();
-                        }
-                    }
-                }
-                CertificateUtil.validatePath(certs,anchors);
-            }
-
-            // this is for computing a digest to check sanity
-            MessageDigest sha1 = MessageDigest.getInstance("SHA1");
-            DigestOutputStream dos = new DigestOutputStream(new NullOutputStream(),sha1);
-
-            // this is for computing a signature
-            Signature sig = Signature.getInstance("SHA1withRSA");
-            sig.initVerify(certs.get(0));
-            SignatureOutputStream sos = new SignatureOutputStream(sig);
-
-            // until JENKINS-11110 fix, UC used to serve invalid digest (and therefore unverifiable signature)
-            // that only covers the earlier portion of the file. This was caused by the lack of close() call
-            // in the canonical writing, which apparently leave some bytes somewhere that's not flushed to
-            // the digest output stream. This affects Jenkins [1.424,1,431].
-            // Jenkins 1.432 shipped with the "fix" (1eb0c64abb3794edce29cbb1de50c93fa03a8229) that made it
-            // compute the correct digest, but it breaks all the existing UC json metadata out there. We then
-            // quickly discovered ourselves in the catch-22 situation. If we generate UC with the correct signature,
-            // it'll cut off [1.424,1.431] from the UC. But if we don't, we'll cut off [1.432,*).
-            //
-            // In 1.433, we revisited 1eb0c64abb3794edce29cbb1de50c93fa03a8229 so that the original "digest"/"signature"
-            // pair continues to be generated in a buggy form, while "correct_digest"/"correct_signature" are generated
-            // correctly.
-            //
-            // Jenkins should ignore "digest"/"signature" pair. Accepting it creates a vulnerability that allows
-            // the attacker to inject a fragment at the end of the json.
-            o.writeCanonical(new OutputStreamWriter(new TeeOutputStream(dos,sos),"UTF-8")).close();
-
-            // did the digest match? this is not a part of the signature validation, but if we have a bug in the c14n
-            // (which is more likely than someone tampering with update center), we can tell
-            String computedDigest = new String(Base64.encode(sha1.digest()));
-            String providedDigest = signature.optString("correct_digest");
-            if (providedDigest==null) {
-                return FormValidation.error("No correct_digest parameter in update center '"+id+"'. This metadata appears to be old.");
-            }
-            if (!computedDigest.equalsIgnoreCase(providedDigest)) {
-                return FormValidation.error("Digest mismatch: "+computedDigest+" vs "+providedDigest+" in update center '"+id+"'");
-            }
-
-            String providedSignature = signature.getString("correct_signature");
-            if (!sig.verify(Base64.decode(providedSignature.toCharArray()))) {
-                return FormValidation.error("Signature in the update center doesn't match with the certificate in update center '"+id+"'");
-            }
-
-            if (warning!=null)  return warning;
-            return FormValidation.ok();
-        } catch (GeneralSecurityException e) {
-            return FormValidation.error(e,"Signature verification failed in the update center '"+id+"'");
-        }
+        return new JSONSignatureValidator("update site '"+id+"'").verifySignature(o);
     }
 
     /**
@@ -294,7 +238,7 @@ public class UpdateSite {
      */
     public boolean isDue() {
         if(neverUpdate)     return false;
-        if(dataTimestamp==-1)
+        if(dataTimestamp == 0)
             dataTimestamp = getDataFile().file.lastModified();
         long now = System.currentTimeMillis();
         
@@ -487,7 +431,7 @@ public class UpdateSite {
      * Is this the legacy default update center site?
      */
     public boolean isLegacyDefault() {
-        return id.equals("default") && url.startsWith("http://hudson-ci.org/") || url.startsWith("http://updates.hudson-labs.org/");
+        return id.equals(UpdateCenter.ID_DEFAULT) && url.startsWith("http://hudson-ci.org/") || url.startsWith("http://updates.hudson-labs.org/");
     }
 
     /**
@@ -516,10 +460,10 @@ public class UpdateSite {
 
         Data(JSONObject o) {
             this.sourceId = (String)o.get("id");
-            if (sourceId.equals("default")) {
-                core = new Entry(sourceId, o.getJSONObject("core"));
-            }
-            else {
+            JSONObject c = o.optJSONObject("core");
+            if (c!=null) {
+                core = new Entry(sourceId, c, url);
+            } else {
                 core = null;
             }
             for(Map.Entry<String,JSONObject> e : (Set<Map.Entry<String,JSONObject>>)o.getJSONObject("plugins").entrySet()) {
@@ -569,10 +513,21 @@ public class UpdateSite {
         public final String url;
 
         public Entry(String sourceId, JSONObject o) {
+            this(sourceId, o, null);
+        }
+
+        Entry(String sourceId, JSONObject o, String baseURL) {
             this.sourceId = sourceId;
             this.name = o.getString("name");
             this.version = o.getString("version");
-            this.url = o.getString("url");
+            String url = o.getString("url");
+            if (!URI.create(url).isAbsolute()) {
+                if (baseURL == null) {
+                    throw new IllegalArgumentException("Cannot resolve " + url + " without a base URL");
+                }
+                url = URI.create(baseURL).resolve(url).toString();
+            }
+            this.url = url;
         }
 
         /**
@@ -644,7 +599,7 @@ public class UpdateSite {
         
         @DataBoundConstructor
         public Plugin(String sourceId, JSONObject o) {
-            super(sourceId, o);
+            super(sourceId, o, UpdateSite.this.url);
             this.wiki = get(o,"wiki");
             this.title = get(o,"title");
             this.excerpt = get(o,"excerpt");
@@ -770,13 +725,19 @@ public class UpdateSite {
          * @param dynamicLoad
          *      If true, the plugin will be dynamically loaded into this Jenkins. If false,
          *      the plugin will only take effect after the reboot.
+         *      See {@link UpdateCenter#isRestartRequiredForCompletion()}
          */
         public Future<UpdateCenterJob> deploy(boolean dynamicLoad) {
             Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
             UpdateCenter uc = Jenkins.getInstance().getUpdateCenter();
             for (Plugin dep : getNeededDependencies()) {
-                LOGGER.log(Level.WARNING, "Adding dependent install of " + dep.name + " for plugin " + name);
-                dep.deploy(dynamicLoad);
+                UpdateCenter.InstallationJob job = uc.getJob(dep);
+                if (job == null || job.status instanceof UpdateCenter.DownloadJob.Failure) {
+                    LOGGER.log(Level.WARNING, "Adding dependent install of " + dep.name + " for plugin " + name);
+                    dep.deploy(dynamicLoad);
+                } else {
+                    LOGGER.log(Level.WARNING, "Dependent install of " + dep.name + " for plugin " + name + " already added, skipping");
+                }
             }
             return uc.addJob(uc.new InstallationJob(this, UpdateSite.this, Jenkins.getAuthentication(), dynamicLoad));
         }
@@ -821,8 +782,4 @@ public class UpdateSite {
     // The name uses UpdateCenter for compatibility reason.
     public static boolean neverUpdate = Boolean.getBoolean(UpdateCenter.class.getName()+".never");
 
-    /**
-     * Off by default until we know this is reasonably working.
-     */
-    public static boolean signatureCheck = true; // Boolean.getBoolean(UpdateCenter.class.getName()+".signatureCheck");
 }
