@@ -45,13 +45,13 @@ import org.kohsuke.stapler.export.ExportedBean;
  * Configuration screen should be defined in <tt>config.jelly</tt>.
  * Within this page, the {@link UserProperty} instance is available
  * as <tt>instance</tt> variable (while <tt>it</tt> refers to {@link User}.
- * See {@link Mailer.UserProperty}'s <tt>config.jelly</tt> for an example.
+ * See {@link UserSearchProperty.UserProperty}'s <tt>config.jelly</tt> for an example.
  *
  *
  * @author Kohsuke Kawaguchi
  */
 @ExportedBean
-public abstract class UserProperty implements Describable<UserProperty>, ExtensionPoint {
+public abstract class UserProperty implements ReconfigurableDescribable<UserProperty>, ExtensionPoint {
     /**
      * The user object that owns this property.
      * This value will be set by the Hudson code.
@@ -76,6 +76,6 @@ public abstract class UserProperty implements Describable<UserProperty>, Extensi
     }
 
     public UserProperty reconfigure(StaplerRequest req, JSONObject form) throws FormException {
-    	return getDescriptor().newInstance(req, form);
+        return form==null ? null : getDescriptor().newInstance(req, form);
     }
 }
