@@ -54,7 +54,9 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
      * This field retains the method name.
      */
     private final String testName;
+    private transient String safeName;
     private final boolean skipped;
+    private final String skippedMessage;
     private final String errorStackTrace;
     private final String errorDetails;
     private transient SuiteResult parent;
@@ -124,6 +126,7 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
         this.parent = parent;
         duration = parseTime(testCase);
         skipped = isMarkedAsSkipped(testCase);
+        skippedMessage = getSkippedMessage(testCase);
         @SuppressWarnings("LeakingThisInConstructor")
         Collection<CaseResult> _this = Collections.singleton(this);
         stdout = possiblyTrimStdio(_this, keepLongStdio, testCase.elementText("system-out"));
@@ -148,7 +151,7 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
         if (middle <= 0) {
             return stdio.toString();
         }
-        return stdio.subSequence(0, HALF_MAX_SIZE) + "...[truncated " + middle + " chars]..." + stdio.subSequence(len - HALF_MAX_SIZE, len);
+        return stdio.subSequence(0, HALF_MAX_SIZE) + "\n...[truncated " + middle + " chars]...\n" + stdio.subSequence(len - HALF_MAX_SIZE, len);
     }
 
     /**
@@ -164,6 +167,7 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
         this.stderr = null;
         this.duration = 0.0f;
         this.skipped = false;
+        this.skippedMessage = null;
     }
     
     public ClassResult getParent() {
@@ -198,6 +202,17 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
         return testCase.element("skipped") != null;
     }
 
+    private static String getSkippedMessage(Element testCase) {
+        String message = null;
+        Element skippedElement = testCase.element("skipped");
+
+        if (skippedElement != null) {
+            message = skippedElement.attributeValue("message");
+        }
+
+        return message;
+    }
+
     public String getDisplayName() {
         return testName;
     }
@@ -232,7 +247,10 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
     /**
      * Gets the version of {@link #getName()} that's URL-safe.
      */
-    public @Override String getSafeName() {
+    public @Override synchronized String getSafeName() {
+        if (safeName != null) {
+            return safeName;
+        }
         StringBuilder buf = new StringBuilder(testName);
         for( int i=0; i<buf.length(); i++ ) {
             char ch = buf.charAt(i);
@@ -240,7 +258,7 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
                 buf.setCharAt(i,'_');
         }
         Collection<CaseResult> siblings = (classResult ==null ? Collections.<CaseResult>emptyList(): classResult.getChildren());
-        return uniquifyName(siblings, buf.toString());
+        return safeName = uniquifyName(siblings, buf.toString());
     }
 
     /**
@@ -456,6 +474,16 @@ public final class CaseResult extends TestResult implements Comparable<CaseResul
         return skipped;
     }
 
+    /**
+     * Provides the reason given for the test being being skipped.
+     * @return the message given for a skipped test if one has been provided, null otherwise.
+     * @since 1.507
+     */
+    @Exported
+    public String getSkippedMessage() {
+        return skippedMessage;
+    }
+
     public SuiteResult getSuiteResult() {
         return parent;
     }
