@@ -27,6 +27,7 @@ import hudson.EnvVars;
 import hudson.FilePath;
 import hudson.maven.reporters.MavenArtifactRecord;
 import hudson.maven.reporters.SurefireArchiver;
+import hudson.maven.reporters.TestFailureDetector;
 import hudson.slaves.WorkspaceList;
 import hudson.slaves.WorkspaceList.Lease;
 import hudson.maven.agent.AbortException;
@@ -48,7 +49,6 @@ import hudson.tasks.Maven.MavenInstallation;
 import hudson.tasks.Publisher;
 import hudson.util.ArgumentListBuilder;
 import hudson.util.DescribableList;
-import hudson.util.IOUtils;
 import org.apache.maven.BuildFailureException;
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.execution.ReactorManager;
@@ -72,7 +72,11 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.logging.Logger;
+import javax.annotation.CheckForNull;
+
+import jenkins.mvn.SettingsProvider;
 
 /**
  * {@link Run} for {@link MavenModule}.
@@ -150,7 +154,7 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
      *      is manually triggered.
      * @see #getModuleSetBuild()
      */
-    public MavenModuleSetBuild getParentBuild() {
+    public @CheckForNull MavenModuleSetBuild getParentBuild() {
         return getParent().getParent().getBuildByNumber(getNumber());
     }
 
@@ -171,7 +175,7 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
      *      is manually removed.
      * @see #getParentBuild()
      */
-    public MavenModuleSetBuild getModuleSetBuild() {
+    public @CheckForNull MavenModuleSetBuild getModuleSetBuild() {
         return getParent().getParent().getNearestOldBuild(getNumber());
     }
 
@@ -295,6 +299,7 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
      */
     private static final class Builder extends MavenBuilder {
         private final MavenBuildProxy buildProxy;
+        private final AtomicBoolean hasTestFailures = new AtomicBoolean();
 
         /**
          * Records of what was executed.
@@ -349,9 +354,15 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
         void postExecute(MavenProject project, MojoInfo info, Exception exception) throws IOException, InterruptedException, AbortException {
             executedMojos.add(new ExecutedMojo(info,System.currentTimeMillis()-startTime));
 
-            for (MavenReporter r : reporters.get(moduleName))
-                if(!r.postExecute(buildProxy,project,info,listener,exception))
+            for (MavenReporter r : reporters.get(moduleName)){
+                if(!r.postExecute(buildProxy,project,info,listener,exception)) {
                     throw new AbortException(r+" failed");
+                } else if (r instanceof TestFailureDetector) {
+                    if(((TestFailureDetector) r).hasTestFailures()) {
+                        hasTestFailures.compareAndSet(false, true);
+                    }
+                }
+            }
         }
 
         @Override
@@ -375,6 +386,11 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
                     throw new AbortException(r+" failed");
         }
 
+        @Override
+        public boolean hasBuildFailures() {
+            return hasTestFailures.get();
+        }
+        
         private static final long serialVersionUID = 1L;
     }
 
@@ -700,21 +716,12 @@ public class MavenBuild extends AbstractMavenBuild<MavenModule,MavenBuild> {
             if(localRepo!=null)
                 // the workspace must be on this node, so getRemote() is safe.
                 margs.add("-Dmaven.repo.local="+localRepo.getRemote());
-
-            if (mms.getAlternateSettings() != null) {
-                if (IOUtils.isAbsolute(mms.getAlternateSettings())) {
-                    margs.add("-s").add(mms.getAlternateSettings());
-                } else {
-                    FilePath mrSettings = getModuleRoot().child(mms.getAlternateSettings());
-                    FilePath wsSettings = getWorkspace().child(mms.getAlternateSettings());
-                    if (!wsSettings.exists() && mrSettings.exists())
-                        wsSettings = mrSettings;
-
-                    margs.add("-s").add(wsSettings.getRemote());
-                }
+            
+            String settingsPath = SettingsProvider.getSettingsRemotePath(mms.getSettings(), MavenBuild.this, listener);
+            if (settingsPath != null) {
+                margs.add("-s").add(settingsPath);
             }
 
-
             margs.add("-f",getModuleRoot().child("pom.xml").getRemote());
             margs.addTokenized(getProject().getGoals());
 
